@@ -1,5 +1,5 @@
 #
-#  Copyright 2002-2015 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 """
 @author: fconejer
 
-PyCOMPSs API - Constraint
+PyCOMPSs API - CONSTRAINT
 =========================
     This file contains the class constraint, needed for the constraint
     definition through the decorator.
@@ -25,6 +25,8 @@ import inspect
 import logging
 import os
 from functools import wraps
+from pycompss.runtime.binding import register_ce
+from pycompss.util.location import i_am_at_master
 
 
 logger = logging.getLogger(__name__)
@@ -39,16 +41,15 @@ class constraint(object):
         # store arguments passed to the decorator
         self.args = args
         self.kwargs = kwargs
-        logger.debug("Init constraint.")
+        logger.debug("Init @constraint decorator...")
         # self = itself.
         # args = not used.
         # kwargs = dictionary with the given constraints.
 
     def __call__(self, func):
 
-        if not inspect.stack()[-2][3] == 'compss_worker':
+        if i_am_at_master():
             # master code
-            from pycompss.runtime.binding import set_constraints
 
             mod = inspect.getmodule(func)
             self.module = mod.__name__    # not func.__module__
@@ -68,49 +69,58 @@ class constraint(object):
                 dirs = path.split(os.path.sep)
                 file_name = os.path.splitext(os.path.basename(path))[0]
                 mod_name = file_name
-                i = -1
 
-                while True:
+                i = len(dirs) - 1
+                while i > 0:
                     new_l = len(path) - (len(dirs[i]) + 1)
                     path = path[0:new_l]
                     if "__init__.py" in os.listdir(path):
-                        # Directory is a package
+                        # directory is a package
                         i -= 1
                         mod_name = dirs[i] + '.' + mod_name
                     else:
                         break
                 self.module = mod_name
 
-            logger.debug("Registering constraints for function %s of module %s"
-                         % (func.__name__, self.module))
-            for key, value in self.kwargs.iteritems():
-                logger.debug("%s -> %s" % (key, value))
-            set_constraints(func.__name__, self.module, self.kwargs)
+            # Include the registering info related to @constraint
+
+            # Retrieve the base coreElement established at @task decorator
+            coreElement = func.__to_register__
+            # Update the core element information with the constraints
+            coreElement.set_implConstraints(self.kwargs)
+            func.__to_register__ = coreElement
+            # Do the task register if I am the top decorator
+            if func.__who_registers__ == __name__:
+                logger.debug("[@CONSTRAINT] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(coreElement)
         else:
             # worker code
             pass
 
+
         @wraps(func)
         def constrained_f(*args, **kwargs):
             # This is executed only when called.
             logger.debug("Executing constrained_f wrapper.")
 
-            # The 'self' for a method function is passed as args[0]
-            slf = args[0]
+            if len(args) > 0:
+                # The 'self' for a method function is passed as args[0]
+                slf = args[0]
 
-            # Replace and store the attributes
-            saved = {}
-            for k, v in self.kwargs.items():
-                if hasattr(slf, k):
-                    saved[k] = getattr(slf, k)
-                    setattr(slf, k, v)
+                # Replace and store the attributes
+                saved = {}
+                for k, v in self.kwargs.items():
+                    if hasattr(slf, k):
+                        saved[k] = getattr(slf, k)
+                        setattr(slf, k, v)
 
             # Call the method
             ret = func(*args, **kwargs)
 
-            # Put things back
-            for k, v in saved.items():
-                setattr(slf, k, v)
+            if len(args) > 0:
+                # Put things back
+                for k, v in saved.items():
+                    setattr(slf, k, v)
 
             return ret
         constrained_f.__doc__ = func.__doc__
