@@ -1,5 +1,5 @@
 #
-#  Copyright 2002-2015 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,8 +13,19 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
+"""
+@author: fconejer
+
+PyCOMPSs Dummy API - Constraint
+===============================
+    This file contains the dummy class constraint used as decorator.
+"""
+
 
 class constraint(object):
+    """
+    Dummy constraint class (decorator style)
+    """
 
     def __init__(self, *args, **kwargs):
         self.args = args
