@@ -1,5 +1,5 @@
 #
-#  Copyright 2002-2015 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,10 +13,19 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
+"""
+@author: scorella
+
+PyCOMPSs Functions: Data generators
+===================================
+    This file defines the common data producing functions.
+"""
+
+
 def sort(iterable, comp=None, key=None, reverse=False):
-    """ Apply function of two arguments cumulatively to the items
-        of data, from left to right, so as to reduce the iterable
-        to a single value.
+    """
+    Apply function of two arguments cumulatively to the items of data, from left to right,
+    so as to reduce the iterable to a single value.
     :param iterable: data.
     :param comp: specifies a custom comparison function of two arguments.
     :param key: specifies a function of one argument that is used to extract a comparison key from each list element.
