@@ -1,5 +1,5 @@
 #
-#  Copyright 2002-2015 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,13 +13,23 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
+"""
+@author: scorella
+
+PyCOMPSs Mathematical Library: Algebra: Mean
+============================================
+    This file contains the arithmetic mean algorithm.
+"""
+
+
 from pycompss.api.task import task
-from pycompss.api.parameter import *
 from pycompss.functions.reduce import mergeReduce
 
 
 def _list_lenght(l):
-    """ Recursive function to get the size of any list """
+    """
+    Recursive function to get the size of any list
+    """
     if l:
         if not isinstance(l[0], list):
             return 1 + _list_lenght(l[1:])
@@ -34,12 +44,12 @@ def _mean(X, n):
 
 
 def mean(X, wait=False):
-    """ Arithmetic mean
+    """
+    Arithmetic mean
     :param X: chunked data
     :param wait: if we want to wait for result. Default False
     :return: mean of X.
     """
-    # chunked data
     n = _list_lenght(X)
     result = mergeReduce(reduce_add, [_mean(x, n) for x in X])
     if wait:
