@@ -1,6 +1,5 @@
 #!/bin/bash
 
-export EXTRAE_HOME=@sub_PREFIXDIR@
-source ${EXTRAE_HOME}/etc/extrae.sh
+source @sub_PREFIXDIR@/etc/extrae.sh
 
 ${EXTRAE_HOME}/bin/extrae -config extrae.xml ./pi_forked
