@@ -20,19 +20,14 @@
  *                                 ---------                                 *
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
-
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/common/events.c $
- | @last_commit: $Date: 2015-04-20 16:32:24 +0200 (lun, 20 abr 2015) $
- | @version:     $Revision: 3250 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
-
-static char UNUSED rcsid[] = "$Id: events.c 3250 2015-04-20 14:32:24Z harald $";
-
 #include "events.h"
 
-#define MPI_EVENTS 86
+#define MPI_EVENTS 100
+
+/******************************************************************************
+ ***  IsMPI
+ ******************************************************************************/
 static unsigned mpi_events[] = {
 	MPI_BSEND_EV, MPI_SSEND_EV, MPI_BARRIER_EV, MPI_BCAST_EV, MPI_SEND_EV,
 	MPI_RECV_EV, MPI_SENDRECV_EV, MPI_SENDRECV_REPLACE_EV, MPI_IBSEND_EV,
@@ -57,13 +52,13 @@ static unsigned mpi_events[] = {
 	MPI_REQUEST_GET_STATUS_COUNTER_EV, MPI_TIME_OUTSIDE_MPI_REQUEST_GET_STATUS_EV,
 	MPI_WIN_CREATE_EV, MPI_WIN_FENCE_EV, MPI_WIN_START_EV, MPI_WIN_FREE_EV,
 	MPI_WIN_POST_EV, MPI_WIN_COMPLETE_EV, MPI_WIN_WAIT_EV,
-	MPI_INTERCOMM_CREATE_EV, MPI_INTERCOMM_MERGE_EV
+	MPI_INTERCOMM_CREATE_EV, MPI_INTERCOMM_MERGE_EV,
+	MPI_IREDUCE_EV, MPI_IALLREDUCE_EV, MPI_IBARRIER_EV, MPI_IBCAST_EV,
+	MPI_IALLTOALL_EV, MPI_IALLTOALLV_EV, MPI_IALLGATHER_EV, MPI_IALLGATHERV_EV,
+	MPI_IGATHER_EV, MPI_IGATHERV_EV, MPI_ISCATTER_EV, MPI_ISCATTERV_EV,
+	MPI_IREDUCESCAT_EV, MPI_ISCAN_EV
  };
 
-/******************************************************************************
- ***  IsMPI
- ******************************************************************************/
-
 unsigned IsMPI (unsigned EvType)
 {
   unsigned evt;
@@ -74,57 +69,25 @@ unsigned IsMPI (unsigned EvType)
   return FALSE;
 }
 
-#define PACX_EVENTS 67
-static unsigned pacx_events[] = {
-	PACX_BSEND_EV, PACX_SSEND_EV, PACX_BARRIER_EV, PACX_BCAST_EV, PACX_SEND_EV,
-	PACX_RECV_EV, PACX_SENDRECV_EV, PACX_SENDRECV_REPLACE_EV, PACX_IBSEND_EV,
-	PACX_ISSEND_EV, PACX_ISEND_EV, PACX_IRECV_EV, PACX_TEST_EV, PACX_WAIT_EV,
-	PACX_CANCEL_EV, PACX_RSEND_EV, PACX_IRSEND_EV, PACX_ALLTOALL_EV,
-	PACX_ALLTOALLV_EV, PACX_ALLREDUCE_EV, PACX_REDUCE_EV, PACX_WAITALL_EV,
-	PACX_PROBE_EV, PACX_IPROBE_EV, PACX_GATHER_EV, PACX_GATHERV_EV,
-	PACX_SCATTER_EV, PACX_SCATTERV_EV, PACX_REDUCESCAT_EV, PACX_SCAN_EV,
-	PACX_WAITANY_EV, PACX_WAITSOME_EV, PACX_FINALIZE_EV, PACX_INIT_EV,
-	PACX_ALLGATHER_EV, PACX_ALLGATHERV_EV, PACX_PERSIST_REQ_EV, PACX_START_EV,
-	PACX_STARTALL_EV, PACX_REQUEST_FREE_EV, PACX_RECV_INIT_EV, PACX_SEND_INIT_EV,
-	PACX_BSEND_INIT_EV, PACX_RSEND_INIT_EV, PACX_SSEND_INIT_EV, PACX_COMM_RANK_EV,
-	PACX_COMM_SIZE_EV, PACX_IPROBE_COUNTER_EV, PACX_TIME_OUTSIDE_IPROBES_EV,
-	PACX_TEST_COUNTER_EV, PACX_FILE_OPEN_EV, PACX_FILE_CLOSE_EV, PACX_FILE_READ_EV,
-	PACX_FILE_READ_ALL_EV, PACX_FILE_WRITE_EV, PACX_FILE_WRITE_ALL_EV, 
-	PACX_FILE_READ_AT_EV, PACX_FILE_READ_AT_ALL_EV, PACX_FILE_WRITE_AT_EV,
-	PACX_FILE_WRITE_AT_ALL_EV, PACX_IRECVED_EV,
-	PACX_COMM_CREATE_EV, PACX_COMM_DUP_EV, PACX_COMM_SPLIT_EV,
-	PACX_CART_CREATE_EV, PACX_CART_SUB_EV, PACX_COMM_FREE_EV };
-
 /******************************************************************************
- ***  IsMPI
+ ***  IsMISC
  ******************************************************************************/
-
-unsigned IsPACX (unsigned EvType)
-{
-  unsigned evt;
-
-  for (evt = 0; evt < PACX_EVENTS; evt++)
-    if (pacx_events[evt] == EvType)
-      return TRUE;
-  return FALSE;
-}
-
-
-#define MISC_EVENTS 41
-static unsigned misc_events[] = {FLUSH_EV, READ_EV, WRITE_EV, APPL_EV, USER_EV,
+#define MISC_EVENTS 61
+static unsigned misc_events[] = {FLUSH_EV, OPEN_EV, FOPEN_EV, READ_EV, WRITE_EV, FREAD_EV, FWRITE_EV, 
+        PREAD_EV, PWRITE_EV, READV_EV, WRITEV_EV, PREADV_EV, PWRITEV_EV, APPL_EV, USER_EV,
 	HWC_DEF_EV, HWC_CHANGE_EV, HWC_EV, TRACING_EV, SET_TRACE_EV, CALLER_EV,
 	CPU_BURST_EV, RUSAGE_EV, MEMUSAGE_EV, MPI_STATS_EV, USRFUNC_EV,
-	SAMPLING_EV, SAMPLING_ADDRESS_EV, SAMPLING_ADDRESS_MEM_LEVEL_EV, SAMPLING_ADDRESS_TLB_LEVEL_EV,
+	SAMPLING_EV, SAMPLING_ADDRESS_LD_EV, SAMPLING_ADDRESS_ST_EV,
+	SAMPLING_ADDRESS_MEM_LEVEL_EV, SAMPLING_ADDRESS_TLB_LEVEL_EV,
 	SAMPLING_ADDRESS_REFERENCE_COST_EV,
 	HWC_SET_OVERFLOW_EV, TRACING_MODE_EV, ONLINE_EV, USER_SEND_EV, USER_RECV_EV,
 	RESUME_VIRTUAL_THREAD_EV, SUSPEND_VIRTUAL_THREAD_EV, TRACE_INIT_EV,
 	REGISTER_STACKED_TYPE_EV, REGISTER_CODELOCATION_TYPE_EV,
-	FORK_EV, WAIT_EV, WAITPID_EV, EXEC_EV, GETCPU_EV, SYSTEM_EV,
-	MALLOC_EV, FREE_EV, CALLOC_EV, REALLOC_EV };
-
-/******************************************************************************
- ***  IsMISC
- ******************************************************************************/
+	FORK_EV, WAIT_EV, WAITPID_EV, EXEC_EV, GETCPU_EV, CPU_EVENT_INTERVAL_EV, SYSTEM_EV,
+	MALLOC_EV, FREE_EV, CALLOC_EV, REALLOC_EV, POSIX_MEMALIGN_EV,
+	MEMKIND_MALLOC_EV, MEMKIND_CALLOC_EV, MEMKIND_REALLOC_EV, MEMKIND_POSIX_MEMALIGN_EV, MEMKIND_FREE_EV,
+	MEMKIND_PARTITION_EV, SYSCALL_EV
+ };
 
 unsigned IsMISC (unsigned EvType)
 {
@@ -140,6 +103,9 @@ unsigned IsMISC (unsigned EvType)
 	return FALSE;
 }
 
+/******************************************************************************
+ ***  IsOpenMP
+ ******************************************************************************/
 #define OMP_EVENTS 28
 static unsigned omp_events[] = { OMPFUNC_EV, PAR_EV, WSH_EV, BARRIEROMP_EV,
 	UNNAMEDCRIT_EV, NAMEDCRIT_EV, WORK_EV, JOIN_EV, OMPSETNUMTHREADS_EV,
@@ -149,10 +115,6 @@ static unsigned omp_events[] = { OMPFUNC_EV, PAR_EV, WSH_EV, BARRIEROMP_EV,
 	TASKGROUP_END_EV, TASKID_EV, OMPT_TASKGROUP_IN_EV, OMPT_DEPENDENCE_EV,
 	OMPT_TASKFUNC_EV, OMP_STATS_EV };
 
-/******************************************************************************
- ***  IsOpenMP
- ******************************************************************************/
-
 unsigned IsOpenMP (unsigned EvType)
 {
   unsigned evt;
@@ -163,6 +125,9 @@ unsigned IsOpenMP (unsigned EvType)
   return FALSE;
 }
 
+/******************************************************************************
+ ***  IsPthread
+ ******************************************************************************/
 #define PTHREAD_EVENTS 14
 static unsigned pthread_events[] = { PTHREAD_CREATE_EV, PTHREAD_JOIN_EV,
 	PTHREAD_DETACH_EV, PTHREAD_FUNC_EV, PTHREAD_RWLOCK_WR_EV, PTHREAD_RWLOCK_RD_EV,
@@ -170,10 +135,6 @@ static unsigned pthread_events[] = { PTHREAD_CREATE_EV, PTHREAD_JOIN_EV,
 	PTHREAD_COND_SIGNAL_EV, PTHREAD_COND_BROADCAST_EV, PTHREAD_COND_WAIT_EV,
 	PTHREAD_EXIT_EV, PTHREAD_BARRIER_WAIT_EV };
 
-/******************************************************************************
- ***  IsPthread
- ******************************************************************************/
-
 unsigned IsPthread (unsigned EvType)
 {
   unsigned evt;
@@ -184,23 +145,30 @@ unsigned IsPthread (unsigned EvType)
   return FALSE;
 }
 
-#define TRT_EVENTS 3
-static unsigned trt_events[] = { TRT_SPAWN_EV, TRT_READ_EV, TRT_USRFUNC_EV };
-
 /******************************************************************************
- ***  IsTRT
+ ***  IsJava
  ******************************************************************************/
+#define JAVA_EVENTS 4
+static unsigned java_events[] = {
+	JAVA_JVMTI_GARBAGECOLLECTOR_EV,
+	JAVA_JVMTI_EXCEPTION_EV,
+	JAVA_JVMTI_OBJECT_ALLOC_EV,
+	JAVA_JVMTI_OBJECT_FREE_EV
+};
 
-unsigned IsTRT (unsigned EvType)
+unsigned IsJava (unsigned EvType)
 {
   unsigned evt;
 
-  for (evt = 0; evt < TRT_EVENTS; evt++)
-    if (trt_events[evt] == EvType)
+  for (evt = 0; evt < JAVA_EVENTS; evt++)
+    if (java_events[evt] == EvType)
       return TRUE;
   return FALSE;
 }
 
+/******************************************************************************
+ ***  IsCUDA
+ ******************************************************************************/
 #define CUDA_EVENTS 14
 static unsigned cuda_events[] = {
 	/* Host events */
@@ -212,10 +180,6 @@ static unsigned cuda_events[] = {
     CUDAKERNEL_GPU_EV, CUDACONFIGKERNEL_GPU_EV, CUDAMEMCPY_GPU_EV,
 	CUDAMEMCPYASYNC_GPU_EV, CUDATHREADBARRIER_GPU_EV };
 
-/******************************************************************************
- ***  IsCUDA
- ******************************************************************************/
-
 unsigned IsCUDA (unsigned EvType)
 {
   unsigned evt;
@@ -226,6 +190,9 @@ unsigned IsCUDA (unsigned EvType)
   return FALSE;
 }
 
+/******************************************************************************
+ ***  IsOpenCL
+ ******************************************************************************/
 #define OPENCL_EVENTS 73
 static unsigned opencl_events[] = {
 	OPENCL_CLCREATEBUFFER_EV, OPENCL_CLCREATECOMMANDQUEUE_EV, 
@@ -275,10 +242,6 @@ static unsigned opencl_events[] = {
 	OPENCL_CLENQUEUEWRITEBUFFERRECT_ASYNC_ACC_EV,
 };
 
-/******************************************************************************
- ***  IsOpenCL
- ******************************************************************************/
-
 unsigned IsOpenCL (unsigned EvType)
 {
   unsigned evt;
@@ -289,7 +252,9 @@ unsigned IsOpenCL (unsigned EvType)
   return FALSE;
 }
 
-
+/******************************************************************************
+ ***  IsOPENSHMEM
+ ******************************************************************************/
 #define OPENSHMEM_EVENTS COUNT_OPENSHMEM_EVENTS
 static unsigned openshmem_events[] = {
   START_PES_EV, SHMEM_MY_PE_EV, _MY_PE_EV, SHMEM_N_PES_EV, _NUM_PES_EV, SHMEM_PE_ACCESSIBLE_EV, SHMEM_ADDR_ACCESSIBLE_EV,
@@ -314,11 +279,6 @@ static unsigned openshmem_events[] = {
   SHMEM_CLEAR_CACHE_LINE_INV_EV, SHMEM_SET_CACHE_LINE_INV_EV, SHMEM_UDCFLUSH_EV, SHMEM_UDCFLUSH_LINE_EV,
 };
 
-
-/******************************************************************************
- ***  IsOPENSHMEM
- ******************************************************************************/
-
 unsigned IsOPENSHMEM (unsigned EvType)
 {
   unsigned evt;
@@ -374,35 +334,6 @@ unsigned IsMPICollective(unsigned EvType)
    return FALSE;
 }
 
-/******************************************************************************
- ***  IsPACXCollective
- ******************************************************************************/
-unsigned IsPACXCollective(unsigned EvType)
-{
-   switch (EvType)
-   {
-      case PACX_BARRIER_EV:
-      case PACX_BCAST_EV:
-      case PACX_ALLTOALL_EV:
-      case PACX_ALLTOALLV_EV:
-      case PACX_REDUCE_EV:
-      case PACX_ALLREDUCE_EV:
-      case PACX_GATHER_EV:
-      case PACX_GATHERV_EV:
-      case PACX_ALLGATHER_EV:
-      case PACX_ALLGATHERV_EV:
-      case PACX_SCATTER_EV:
-      case PACX_SCATTERV_EV:
-      case PACX_REDUCESCAT_EV:
-      case PACX_SCAN_EV:
-         return TRUE;
-      default:
-         return FALSE;
-   }
-   return FALSE;
-}
-
-
 
 /******************************************************************************
  ***  getEventType
@@ -430,16 +361,6 @@ EventType_t getEventType (unsigned EvType, unsigned *Type)
 		*Type = PTHREAD_TYPE;
 		return TRUE;
 	}
-	else if (IsTRT (EvType))
-	{
-		*Type = TRT_TYPE;
-		return TRUE;
-	}
-	else if (IsPACX(EvType))
-	{
-		*Type = PACX_TYPE;
-		return TRUE;
-	}
 	else if (IsCUDA(EvType))
 	{
 		*Type = CUDA_TYPE;
@@ -455,16 +376,16 @@ EventType_t getEventType (unsigned EvType, unsigned *Type)
 		*Type = OPENSHMEM_TYPE;
 		return TRUE;
 	}
+	else if (IsJava(EvType))
+	{
+		*Type = JAVA_TYPE;
+		return TRUE;
+	}
 	else if (EvType == MPI_ALIAS_COMM_CREATE_EV)
 	{
 		*Type = MPI_COMM_ALIAS_TYPE;
 		return TRUE;
 	}
-	else if (EvType == PACX_ALIAS_COMM_CREATE_EV)
-	{
-		*Type = PACX_COMM_ALIAS_TYPE;
-		return TRUE;
-	}
 	return FALSE;
 }
 
