@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/dimemas/mpi_trf_semantics.c $
- | @last_commit: $Date: 2012-10-25 13:05:07 +0200 (jue, 25 oct 2012) $
- | @version:     $Revision: 1311 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: mpi_trf_semantics.c 1311 2012-10-25 11:05:07Z harald $";
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
