@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/API/misc_wrapper.c $
- | @last_commit: $Date: 2015-06-04 13:21:47 +0200 (jue, 04 jun 2015) $
- | @version:     $Revision: 3349 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: misc_wrapper.c 3349 2015-06-04 11:21:47Z harald $";
-
 #ifdef HAVE_STDIO_H
 # include <stdio.h>
 #endif
@@ -74,6 +67,7 @@ static char UNUSED rcsid[] = "$Id: misc_wrapper.c 3349 2015-06-04 11:21:47Z hara
 #include "misc_wrapper.h"
 #include "common_hwc.h"
 #include "threadinfo.h"
+#include "sampling-common.h"
 
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
@@ -156,12 +150,12 @@ void Extrae_set_options_Wrapper (int options)
 	Trace_HWC_Enabled = options & EXTRAE_HWC_OPTION;
 	tracejant_mpi     = options & EXTRAE_MPI_OPTION;   
 	tracejant_omp     = options & EXTRAE_OMP_OPTION;   
-	tracejant_pthread = options & EXTRAE_PTHREAD_OPTION;
+	Extrae_set_pthread_tracing (options & EXTRAE_PTHREAD_OPTION);
 	tracejant_hwc_mpi = options & EXTRAE_MPI_HWC_OPTION;
 	tracejant_hwc_omp = options & EXTRAE_OMP_HWC_OPTION;   
-	tracejant_hwc_pthread = options & EXTRAE_PTHREAD_HWC_OPTION;
+	Extrae_set_pthread_hwc_tracing (options & EXTRAE_PTHREAD_HWC_OPTION);
 	tracejant_hwc_uf  = options & EXTRAE_UF_HWC_OPTION;
-	setSamplingEnabled (options & EXTRAE_SAMPLING_OPTION);
+	Extrae_setSamplingEnabled (options & EXTRAE_SAMPLING_OPTION);
 }
 
 void Extrae_getrusage_set_to_0_Wrapper (UINT64 time)
@@ -259,10 +253,11 @@ void Extrae_memusage_Wrapper (void)
 #endif
 }
 
-void Extrae_user_function_Wrapper (unsigned enter)
+UINT64 Extrae_user_function_Wrapper (unsigned enter)
 {
-	UINT64 ip = (enter)?get_caller(4):EMPTY;
+	UINT64 ip = (enter)?Extrae_get_caller(4):EMPTY;
 	TRACE_EVENTANDCOUNTERS (LAST_READ_TIME, USRFUNC_EV, ip, tracejant_hwc_uf);
+	return ip;
 }
 
 void Extrae_function_from_address_Wrapper (extrae_type_t type, void *address)
@@ -287,7 +282,7 @@ static void Generate_Task_File_List (void)
 	char hostname[1024];
 	char tmp_line[1024];
 
-	sprintf (tmpname, "%s/%s.mpits", final_dir, appl_name);
+	sprintf (tmpname, "%s/%s%s", final_dir, appl_name, EXT_MPITS);
 
 	filedes = open (tmpname, O_RDWR | O_CREAT | O_TRUNC, 0644);
 	if (filedes < 0)
@@ -328,8 +323,9 @@ void Extrae_init_tracing (int forked)
 	if (!Backend_preInitialize (TASKID, Extrae_get_num_tasks(), config_file, forked))
 		return;
 
-	/* Generate a tentative file list if we don't reuse a previous execution */
-	if (getenv ("EXTRAE_APPEND_PID") == NULL)
+	/* Generate a tentative file list if we don't reuse a previous execution through
+	   Extrae cmd commands */
+	if (!Extrae_getAppendingEventsToGivenPID(NULL))
 		Generate_Task_File_List();
 
 	/* Take the time */
@@ -363,8 +359,6 @@ void Extrae_init_Wrapper (void)
 			previous = "API";
 		else if (Extrae_is_initialized_Wrapper() == EXTRAE_INITIALIZED_MPI_INIT)
 			previous = "MPI";
-		else if (Extrae_is_initialized_Wrapper() == EXTRAE_INITIALIZED_PACX_INIT)
-			previous = "PACX";
 		else if (Extrae_is_initialized_Wrapper() == EXTRAE_INITIALIZED_SHMEM_INIT)
 			previous = "SHMEM";
 
@@ -378,25 +372,25 @@ void Extrae_fini_Wrapper (void)
 	/* Finalize only if its initialized by Extrae_init call */
 	if (Extrae_is_initialized_Wrapper() == EXTRAE_INITIALIZED_EXTRAE_INIT)
 	{
-#if !defined(IS_CELL_MACHINE)
-		/* If the application is MPI/PACX the MPI/PACX wrappers are responsible
+		/* If the application is MPI the MPI wrappers are responsible
 		   for gathering and generating the .MPITS file*/
-		if (!Extrae_get_ApplicationIsMPI() && !Extrae_get_ApplicationIsPACX() && !Extrae_get_ApplicationIsSHMEM())
-			if (getenv ("EXTRAE_APPEND_PID") == NULL)
+		if (!Extrae_get_ApplicationIsMPI() && !Extrae_get_ApplicationIsSHMEM())
+			/* If we are appending into the file (i.e. using the cmd-line) don't
+			   change the already existing .mpits file */
+			if (!Extrae_getAppendingEventsToGivenPID(NULL))
 				Generate_Task_File_List();
-#endif
 
 		/* Finalize tracing library */
 		Backend_Finalize ();
 
 		/* Call additional code to finalize the task including
-	     MPI_Finalize, PACX_Finalize,... */
+	     MPI_Finalize,... */
 		Extrae_finalize_task();
-
-		Extrae_set_is_initialized (EXTRAE_NOT_INITIALIZED);
 	}
 }
 
+/* This will be called by the atexit() hook. If this happens and the app is MPI we
+   warn about a finalization that does not occur through MPI_Finalize */
 void Extrae_fini_last_chance_Wrapper (void)
 {
 	/* Finalize independently from who did the initialization ! */
@@ -406,21 +400,20 @@ void Extrae_fini_last_chance_Wrapper (void)
 		if (Extrae_is_initialized_Wrapper() == EXTRAE_INITIALIZED_MPI_INIT)
 			fprintf (stderr, PACKAGE_NAME": Warning! MPI task %d application did not terminate using MPI_Finalize! Review your application code.\n", TASKID);
 
-#if !defined(IS_CELL_MACHINE)
-		/* If the application is MPI/PACX the MPI/PACX wrappers are responsible
+		/* If the application is MPI the MPI wrappers are responsible
 		   for gathering and generating the .MPITS file*/
-		if (!Extrae_get_ApplicationIsMPI() && !Extrae_get_ApplicationIsPACX() && !Extrae_get_ApplicationIsSHMEM())
-			Generate_Task_File_List();
-#endif
+		if (!Extrae_get_ApplicationIsMPI() && !Extrae_get_ApplicationIsSHMEM())
+			/* If we are appending into the file (i.e. using the cmd-line) don't
+			   change the already existing .mpits file */
+			if (!Extrae_getAppendingEventsToGivenPID(NULL))
+				Generate_Task_File_List();
 
 		/* Finalize tracing library */
 		Backend_Finalize ();
 
 		/* Call additional code to finalize the task including
-		MPI_Finalize, PACX_Finalize,... */
+		MPI_Finalize, ... */
 		Extrae_finalize_task();
-
-		Extrae_set_is_initialized (EXTRAE_NOT_INITIALIZED);
 	}
 }
 
@@ -461,7 +454,7 @@ void Extrae_emit_CombinedEvents_Wrapper (struct extrae_CombinedEvents *ptr)
 	*/
 	if (ptr->UserFunction != EXTRAE_USER_FUNCTION_NONE)
 	{
-		UINT64 ip = (ptr->UserFunction == EXTRAE_USER_FUNCTION_ENTER)?get_caller(4):EMPTY;
+		UINT64 ip = (ptr->UserFunction == EXTRAE_USER_FUNCTION_ENTER)?Extrae_get_caller(4):EMPTY;
 #if USE_HARDWARE_COUNTERS
 		int EmitHWC = (!ptr->HardwareCounters && tracejant_hwc_uf);
 		TRACE_EVENTANDCOUNTERS (LAST_READ_TIME, USRFUNC_EV, ip, EmitHWC);
@@ -473,7 +466,7 @@ void Extrae_emit_CombinedEvents_Wrapper (struct extrae_CombinedEvents *ptr)
 	/* Now emit the callers */
 	if (ptr->Callers)
 	{
-		trace_callers (LAST_READ_TIME, 4, CALLER_MPI);
+		Extrae_trace_callers (LAST_READ_TIME, 4, CALLER_MPI);
 	}
 
 	/* Finally emit user communications */
