@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/API/wrapper.c $
- | @last_commit: $Date: 2015-06-12 11:13:16 +0200 (vie, 12 jun 2015) $
- | @version:     $Revision: 3359 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: wrapper.c 3359 2015-06-12 09:13:16Z harald $";
-
 #ifdef HAVE_STDIO_H
 # include <stdio.h>
 #endif
@@ -48,14 +41,6 @@ static char UNUSED rcsid[] = "$Id: wrapper.c 3359 2015-06-12 09:13:16Z harald $"
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
-#if defined(MPI_SUPPORT)
-# ifdef HAVE_MPI_H
-#  include <mpi.h>
-# endif
-#endif
-#if defined(PACX_SUPPORT)
-# include <pacx.h>
-#endif
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
@@ -105,18 +90,10 @@ static char UNUSED rcsid[] = "$Id: wrapper.c 3359 2015-06-12 09:13:16Z harald $"
 	extern int sched_getcpu(void);
 #endif 
 
-#if defined(IS_CELL_MACHINE)
-# include "defaults.h"
-# include "cell_wrapper.h"
-#endif
-
 #include "wrapper.h"
 #if defined(MPI_SUPPORT)
 # include "mpi_wrapper.h"
 #endif
-#if defined(PACX_SUPPORT)
-# include "pacx_wrapper.h"
-#endif
 #include "misc_wrapper.h"
 #include "fork_wrapper.h"
 #include "clock.h"
@@ -132,6 +109,9 @@ static char UNUSED rcsid[] = "$Id: wrapper.c 3359 2015-06-12 09:13:16Z harald $"
 #if defined(OMP_SUPPORT)
 # include "omp_probe.h"
 # include "omp_wrapper.h"
+# if defined(OMPT_INSTRUMENTATION)
+#  include "ompt-wrapper.h"
+# endif
 #endif
 #include "trace_buffers.h"
 #include "timesync.h"
@@ -155,7 +135,11 @@ static char UNUSED rcsid[] = "$Id: wrapper.c 3359 2015-06-12 09:13:16Z harald $"
 # include "options.h"
 #endif
 
-#include "sampling.h"
+#include "sampling-common.h"
+#include "sampling-timer.h"
+#if defined(ENABLE_PEBS_SAMPLING)
+# include "sampling-intel-pebs.h"
+#endif
 #include "debug.h"
 #include "threadinfo.h"
 #if defined(PTHREAD_SUPPORT)
@@ -166,6 +150,8 @@ static char UNUSED rcsid[] = "$Id: wrapper.c 3359 2015-06-12 09:13:16Z harald $"
 #include "malloc_wrapper.h"
 #include "io_probe.h"
 #include "io_wrapper.h"
+#include "syscall_probe.h"
+#include "syscall_wrapper.h"
 #include "taskid.h"
 
 int Extrae_Flush_Wrapper (Buffer_t *buffer);
@@ -201,6 +187,13 @@ void setRequestedIOInstrumentation (int b)
 	requestedIOInstrumentation = b;
 }
 
+static int requestedSysCallInstrumentation = FALSE;
+
+void setRequestedSysCallInstrumentation (int b)
+{
+	requestedSysCallInstrumentation = b;
+}
+
 static void Backend_Finalize_close_mpits (pid_t pid, int thread, int append);
 
 /***** Variable global per saber si en un moment donat cal tracejar ******/
@@ -212,12 +205,6 @@ int tracejant_mpi = TRUE;
 /***** Variable global per saber si MPI s'ha de tracejar amb hwc *********/
 int tracejant_hwc_mpi = TRUE;
 
-/***** Global variable to control whether PACX must be instrumented ******/
-int tracejant_pacx = TRUE;
-
-/***** Global variable to control whether HWC must be gathered at PACX ***/
-int tracejant_hwc_pacx = FALSE;
-
 /***** Variable global per saber si OpenMP s'ha de tracejar **************/
 int tracejant_omp = TRUE;
 
@@ -227,9 +214,21 @@ int tracejant_hwc_omp = TRUE;
 /***** Variable global per saber si pthread s'ha de tracejar **************/
 int tracejant_pthread = TRUE;
 
+void Extrae_set_pthread_tracing (int b)
+{ tracejant_pthread = b; }
+
+int Extrae_get_pthread_tracing (void)
+{ return tracejant_pthread; }
+
 /***** Variable global per saber si pthread s'ha de tracejar amb hwc ******/
 int tracejant_hwc_pthread = TRUE;
 
+void Extrae_set_pthread_hwc_tracing (int b)
+{ tracejant_hwc_pthread = b; }
+
+int Extrae_get_pthread_hwc_tracing (void)
+{ return tracejant_hwc_pthread; }
+
 /***** Variable global per saber si UFs s'han de tracejar amb hwc ********/
 int tracejant_hwc_uf = TRUE;
 
@@ -247,8 +246,6 @@ int tracejant_memusage = FALSE;
 /**** no generen trasa ***************************************************/
 int *TracingBitmap = NULL;
 
-int mpit_gathering_enabled;
-
 /*************************************************************************/
 
 /** Variable global per saber si en general cal interceptar l'aplicacio **/
@@ -267,14 +264,18 @@ char PROGRAM_NAME[256];
 
 unsigned long long last_mpi_exit_time = 0;
 unsigned long long last_mpi_begin_time = 0;
-unsigned long long last_pacx_exit_time = 0;
-unsigned long long last_pacx_begin_time = 0;
 
 /* Control del temps de traceig */
 unsigned long long initTracingTime = 0;
 unsigned long long MinimumTracingTime;
 int hasMinimumTracingTime = FALSE;
 
+/* CPU events emission frequency */
+unsigned long long MinimumCPUEventTime = 0;
+unsigned short AlwaysEmitCPUEvent = 0;
+iotimer_t *LastCPUEmissionTime = NULL;
+int *LastCPUEvent = NULL;
+
 unsigned long long WantedCheckControlPeriod = 0;
 
 /******* Variable amb l'estructura que a SGI es guarda al PRDA *******/
@@ -305,6 +306,26 @@ char appl_name[APPL_NAME_LENGTH];
 char final_dir[TMP_DIR];
 char tmp_dir[TMP_DIR];
 
+
+/* Checks if there is a CPU event waiting to be emitted */
+int PENDING_TRACE_CPU_EVENT(int thread_id, iotimer_t current_time)
+{
+	if (MinimumCPUEventTime > 0)
+	{
+		if (LastCPUEmissionTime[thread_id] == 0)
+		{
+			LastCPUEmissionTime[thread_id] = current_time;
+		}
+		else if ((current_time - LastCPUEmissionTime[thread_id]) >  MinimumCPUEventTime)
+		{
+			LastCPUEmissionTime[thread_id] = current_time;
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 /* HSG
 
  MN GPFS optimal files per directories is 512.
@@ -348,9 +369,35 @@ char *Extrae_Get_TemporalDirNoTask (void)
 #endif
 
 /* Know if the run is controlled by a creation of a file  */
-char ControlFileName[TMP_DIR];
-int CheckForControlFile = FALSE;
-int CheckForGlobalOpsTracingIntervals = FALSE;
+static char ControlFileName[TMP_DIR];
+static int CheckForControlFile = FALSE;
+static int CheckForGlobalOpsTracingIntervals = FALSE;
+
+int Extrae_getCheckControlFile (void)
+{
+	return CheckForControlFile;
+}
+char *Extrae_getCheckControlFileName (void)
+{
+	return ControlFileName;
+}
+int Extrae_getCheckForGlobalOpsTracingIntervals (void)
+{
+	return CheckForGlobalOpsTracingIntervals;
+}
+
+void Extrae_setCheckControlFile (int b)
+{
+	CheckForControlFile = b;
+}
+void Extrae_setCheckControlFileName (const char *f)
+{
+	strcpy (ControlFileName, f);
+}
+void Extrae_setCheckForGlobalOpsTracingIntervals (int b)
+{
+	CheckForGlobalOpsTracingIntervals = b;
+}
 
 int circular_buffering = 0;
 event_t *circular_HEAD;
@@ -361,6 +408,22 @@ static void Extrae_getExecutableInfo (void);
 int MergeAfterTracing = FALSE;
 #endif
 
+static int AppendingEventsToGivenPID = FALSE;
+static int AppendingEventsToGivenPID_PID = 0;
+
+void Extrae_setAppendingEventsToGivenPID (int pid)
+{
+	AppendingEventsToGivenPID = TRUE;
+	AppendingEventsToGivenPID_PID = pid;
+}
+
+int Extrae_getAppendingEventsToGivenPID (int *pid)
+{
+	if (pid != NULL)
+		*pid = AppendingEventsToGivenPID_PID;
+	return AppendingEventsToGivenPID;
+}
+
 static int extrae_initialized = FALSE;
 
 int EXTRAE_ON (void) { return mpitrace_on; }
@@ -424,7 +487,13 @@ static void Extrae_BG_gettopology (int enter, UINT64 timestamp)
 void Extrae_AnnotateCPU (UINT64 timestamp)
 {
 #if defined(HAVE_SCHED_GETCPU)
-    TRACE_EVENT (timestamp, GETCPU_EV, sched_getcpu());
+    int cpu = sched_getcpu();
+
+    if (cpu != LastCPUEvent[THREADID] || AlwaysEmitCPUEvent)
+    {
+        LastCPUEvent[THREADID] = cpu;
+        TRACE_EVENT (timestamp, GETCPU_EV, cpu);
+    }
 #else
     UNREFERENCED_PARAMETER(timestamp);
 #endif
@@ -446,10 +515,9 @@ char * Get_ApplName (void)
 }
 
 /******************************************************************************
- *** Store whether the app is MPI and/or PACX
+ *** Store whether the app is MPI
  ******************************************************************************/
 static int Extrae_Application_isMPI = FALSE;
-static int Extrae_Application_isPACX = FALSE;
 static int Extrae_Application_isSHMEM = FALSE;
 
 int Extrae_get_ApplicationIsMPI (void)
@@ -457,21 +525,11 @@ int Extrae_get_ApplicationIsMPI (void)
 	return Extrae_Application_isMPI;
 }
 
-int Extrae_get_ApplicationIsPACX (void)
-{
-	return Extrae_Application_isPACX;
-}
-
 void Extrae_set_ApplicationIsMPI (int b)
 {
 	Extrae_Application_isMPI = b;
 }
 
-void Extrae_set_ApplicationIsPACX (int b)
-{
-	Extrae_Application_isPACX = b;
-}
-
 int Extrae_get_ApplicationIsSHMEM (void)
 {
         return Extrae_Application_isSHMEM;
@@ -532,7 +590,7 @@ void Backend_createExtraeDirectory (int taskid, int Temporal)
 
 static int read_environment_variables (int me)
 {
-#if defined(MPI_SUPPORT) || defined(PACX_SUPPORT)
+#if defined(MPI_SUPPORT)
 	char *mpi_callers;
 #endif
   char *dir, *str, *res_cwd;
@@ -623,19 +681,6 @@ static int read_environment_variables (int me)
 			TMODE_setBurstsStatistics (DISABLED);
 		}
 	}
-#elif defined(PACX_SUPPORT)
-	/* Collect MPI statistics in the library? */
-	if ((str = getenv ("EXTRAE_PACX_STATISTICS")) != NULL)
-	{
-		if (strcmp(str, "1") == 0)
-		{
-			TMODE_setBurstsStatistics (ENABLED);
-		}
-		else
-		{
-			TMODE_setBurstsStatistics (DISABLED);
-		}
-	}
 #endif
 
 	/*
@@ -679,18 +724,18 @@ static int read_environment_variables (int me)
 	/* EXTRAE_CONTROL_FILE, activates the tracing when this file is created */
 	if ((file = getenv ("EXTRAE_CONTROL_FILE")) != NULL)
 	{
-		CheckForControlFile = TRUE;
-		strcpy (ControlFileName, file);
+		Extrae_setCheckControlFile (TRUE);
+		Extrae_setCheckControlFileName (file);
 		if (me == 0)
-			fprintf (stdout, PACKAGE_NAME": Control file is %s.\n          Tracing will be disabled until the file exists\n", ControlFileName);
+			fprintf (stdout, PACKAGE_NAME": Control file is %s.\n          Tracing will be disabled until the file exists\n", file);
 	}
 	else
-		CheckForControlFile = FALSE;
+		Extrae_setCheckControlFile (FALSE);
 
 	/* EXTRAE_CONTROL_GLOPS, activates the tracing on a global op series */
 	if ((str = getenv ("EXTRAE_CONTROL_GLOPS")) != NULL)
 	{
-		CheckForGlobalOpsTracingIntervals = TRUE;
+		Extrae_setCheckForGlobalOpsTracingIntervals(TRUE);
 		Parse_GlobalOps_Tracing_Intervals (str);
 	}
 
@@ -749,21 +794,6 @@ static int read_environment_variables (int me)
 	/* Control if the user wants to add information about MPI caller routines */
 	mpi_callers = getenv ("EXTRAE_MPI_CALLER");
 	if (mpi_callers != NULL) Parse_Callers (me, mpi_callers, CALLER_MPI);
-#elif defined(PACX_SUPPORT)
-	/* Control if the user wants to add information about MPI caller routines */
-	mpi_callers = getenv ("EXTRAE_PACX_CALLER");
-	if (mpi_callers != NULL) Parse_Callers (me, mpi_callers, CALLER_MPI);
-#endif
-
-#if defined(MPI_SUPPORT)
-	/* Check if we must gather all the MPIT files into one target (MASTER) node */
-	str = getenv("EXTRAE_GATHER_MPITS");
-	if ((str != NULL) && (strcmp(str, "1") == 0))
-	{
-		mpit_gathering_enabled = TRUE;
-		if (me == 0)
-			fprintf (stdout, PACKAGE_NAME": All MPIT files will be gathered at the end of the execution.\n");
-	}
 #endif
 
 	/* Check if the buffer must be treated as a circular buffer instead a linear buffer with many flushes */
@@ -792,15 +822,6 @@ static int read_environment_variables (int me)
 			fprintf (stdout, PACKAGE_NAME": MPI calls are NOT traced.\n");
   	tracejant_mpi = FALSE;
 	}
-#elif defined(PACX_SUPPORT)
-	/* Check if the PACX must be disabled */
-	str = getenv ("EXTRAE_DISABLE_PACX");
-	if (str != NULL && (strcmp (str, "1") == 0))
-	{
-		if (me == 0)
-			fprintf (stdout, PACKAGE_NAME": PACX calls are NOT traced.\n");
-  	tracejant_mpi = FALSE;
-	}
 #endif
 
 #if defined(MPI_SUPPORT)
@@ -814,15 +835,6 @@ static int read_environment_variables (int me)
 	}
 	else
 		tracejant_hwc_mpi = FALSE;
-#elif defined(PACX_SUPPORT)
-	/* HWC must be gathered at PACX? */
-	str = getenv ("EXTRAE_PACX_COUNTERS_ON");
-	if (str != NULL && (strcmp (str, "1") == 0))
-	{
-		if (me == 0)
-			fprintf (stdout, PACKAGE_NAME": HWC reported in the PACX calls.\n");
-		tracejant_hwc_mpi = TRUE;
-	}
 #endif
 
 	/* Enable rusage information? */
@@ -1003,71 +1015,6 @@ static int read_environment_variables (int me)
 		Parse_Callers (me, getenv("EXTRAE_SAMPLING_CALLER"), CALLER_SAMPLING);
 #endif
 
-#if defined(IS_CELL_MACHINE)
-
-# ifndef SPU_USES_WRITE
-	/* Configure DMA channel for the transferences */
-	str = getenv("EXTRAE_SPU_DMA_CHANNEL");
-	if (str == (char *)NULL) {
-		spu_dma_channel = DEFAULT_DMA_CHANNEL;
-	}
-	else {
-		spu_dma_channel = atoi(str);
-	}
-	if ((spu_dma_channel < 0) || (spu_dma_channel > 31))
-	{
-		if (TASKID == 0)
-			fprintf (stderr, PACKAGE_NAME": Invalid DMA channel '%d'. Using default channel '%d'.\n", spu_dma_channel, DEFAULT_DMA_CHANNEL);
-		spu_dma_channel = DEFAULT_DMA_CHANNEL;
-	}
-# else
-	if (getenv("EXTRAE_SPU_DMA_CHANNEL") != NULL)
-		if (TASKID == 0)
-			fprintf (stdout, PACKAGE_NAME": SPUs will write directly to disk. Ignoring EXTRAE_SPU_DMA_CHANNEL\n");
-# endif /* SPU_USES_WRITE */
-
-	/* Configure the buffer size for each SPU */
-	str = getenv("EXTRAE_SPU_BUFFER_SIZE");
-	if (str == (char *)NULL) {
-		spu_buffer_size = DEFAULT_SPU_BUFFER_SIZE;
-	}
-	else {
-		spu_buffer_size = atoi(str);
-	}
-	if (spu_buffer_size < 10)
-	{
-		if (TASKID == 0)
-			fprintf (stderr, PACKAGE_NAME": SPU tracing buffer size '%d' too small. Using default SPU buffer size '%d'.\n", spu_buffer_size, DEFAULT_SPU_BUFFER_SIZE);
-		spu_buffer_size = DEFAULT_SPU_BUFFER_SIZE;
-	}
-	else
-	{
-		if (TASKID == 0)
-			fprintf (stdout, PACKAGE_NAME": SPU tracing buffer size is %d events.\n", spu_buffer_size);
-	}
-
-	/* Limit the total size of tracing of each spu */
-	str = getenv ("EXTRAE_SPU_FILE_SIZE");
-	if (str == (char *)NULL) {
-		spu_file_size = DEFAULT_SPU_FILE_SIZE;
-	}
-	else {
-	spu_file_size = atoi(str);
-	}
-	if (spu_file_size < 1)
-	{
-		if (TASKID == 0)
-			fprintf (stderr, PACKAGE_NAME": SPU tracing buffer size '%d' too small. Using default SPU buffer size '%d'.\n", spu_file_size, DEFAULT_SPU_FILE_SIZE);
-		spu_file_size = DEFAULT_SPU_FILE_SIZE;
-	}
-	else
-	{
-		if (TASKID == 0)
-			fprintf (stdout, PACKAGE_NAME": SPU tracing file size limit is %d mbytes.\n", spu_file_size);
-	}
-
-#endif /* IS_CELL_MACHINE */
-
 	return 1;
 }
 
@@ -1145,12 +1092,35 @@ void Parse_Callers (int me, char * mpi_callers, int type)
          Caller_Count[type]++;
       }                                                                           
    }                                                                              
-   if (Caller_Count[type] > 0 && me == 0) {
-      fprintf(stdout, PACKAGE_NAME": Tracing %d level(s) of %s callers: [ ",
-				 	Caller_Count[type], (CALLER_MPI==type)?"MPI":"Sampling");
-      for (i=0; i<Caller_Deepness[type]; i++) {
-         if (Trace_Caller[type][i]) fprintf(stdout, "%d ", i+1);
-      }
+	if (Caller_Count[type] > 0 && me == 0)
+	{	
+		const char *s;
+		const char *s_mpi = "MPI";
+		const char *s_sampling = "Sampling";
+		const char *s_malloc = "Dynamic-Memory";
+		const char *s_io = "Input/Output";
+		const char *s_syscall = "System Calls";
+		const char *s_unknown = "unknown?";
+
+		if (CALLER_MPI == type)
+			s = s_mpi;
+		else if (CALLER_SAMPLING == type)
+			s = s_sampling;
+		else if (CALLER_DYNAMIC_MEMORY == type)
+			s = s_malloc;
+		else if (CALLER_IO == type)
+			s = s_io;
+		else if (CALLER_SYSCALL == type)
+			s = s_syscall;
+		else
+			s = s_unknown;
+
+		fprintf(stdout, PACKAGE_NAME": Tracing %d level(s) of %s callers: [ ",
+		  Caller_Count[type], s);
+
+		for (i=0; i<Caller_Deepness[type]; i++)
+			if (Trace_Caller[type][i])
+				fprintf(stdout, "%d ", i+1);
       fprintf(stdout, "]\n");
    }
 }
@@ -1321,6 +1291,8 @@ static int Allocate_buffer_and_file (int thread_id, int forked)
 	if (forked)
 		Buffer_Free (TracingBuffer[thread_id]);
 
+	LastCPUEmissionTime[thread_id] = 0;
+	LastCPUEvent[thread_id] = 0;
 	TracingBuffer[thread_id] = new_Buffer (buffer_size, tmp_file, TRUE);
 	if (TracingBuffer[thread_id] == NULL)
 	{
@@ -1390,6 +1362,8 @@ static int Allocate_buffers_and_files (int world_size, int num_threads, int fork
 	if (!forked)
 	{
 		xmalloc(TracingBuffer, num_threads * sizeof(Buffer_t *));
+		xmalloc(LastCPUEmissionTime, num_threads * sizeof(iotimer_t));
+		xmalloc(LastCPUEvent, num_threads * sizeof(int));
 #if defined(SAMPLING_SUPPORT)
 		xmalloc(SamplingBuffer, num_threads * sizeof(Buffer_t *));
 #endif
@@ -1411,6 +1385,8 @@ static int Reallocate_buffers_and_files (int new_num_threads)
 	int i;
 
 	xrealloc(TracingBuffer, TracingBuffer, new_num_threads * sizeof(Buffer_t *));
+	xrealloc(LastCPUEmissionTime, LastCPUEmissionTime, new_num_threads * sizeof(iotimer_t));
+	xrealloc(LastCPUEvent, LastCPUEvent, new_num_threads * sizeof(int));
 #if defined(SAMPLING_SUPPORT)
 	xrealloc(SamplingBuffer, SamplingBuffer, new_num_threads * sizeof(Buffer_t *));
 #endif
@@ -1499,12 +1475,13 @@ int Backend_ispThreadFinished (int threadid)
 
 void Backend_Flush_pThread (pthread_t t)
 {
-	unsigned u, max = Backend_getNumberOfThreads();
+	unsigned u;
 
-	for (u = 0; u < max; u++)
-	{
+	for (u = 0; u < get_maximum_NumOfThreads(); u++)
 		if (pThreads[u] == t)
 		{
+			pThreads[u] = (pthread_t)0; // This slot won't be used in the future
+
 			Buffer_Flush(TRACING_BUFFER(u));
 			Backend_Finalize_close_mpits (getpid(), u, FALSE);
 
@@ -1514,11 +1491,8 @@ void Backend_Flush_pThread (pthread_t t)
 			Buffer_Free (SAMPLING_BUFFER(u));
 			SAMPLING_BUFFER(u) = NULL;
 #endif
-
-			pThreads[u] = (pthread_t)0;
 			break;
 		}
-	}
 }
 
 void Backend_CreatepThreadIdentifier (void)
@@ -1566,7 +1540,7 @@ void Backend_setNumTentativeThreads (int numofthreads)
  * Backend_preInitialize :
  ******************************************************************************/
 
-int Backend_preInitialize (int me, int world_size, char *config_file, int forked)
+int Backend_preInitialize (int me, int world_size, const char *config_file, int forked)
 {
 	unsigned u;
 	int runningInDynInst = FALSE;
@@ -1579,9 +1553,14 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 #if USE_HARDWARE_COUNTERS
 	int set;
 #endif
-	int appending = getenv ("EXTRAE_APPEND_PID") != NULL;
 	char hostname[1024];
 
+	if (getenv("EXTRAE_APPEND_PID") != NULL)
+		Extrae_setAppendingEventsToGivenPID (atoi(getenv("EXTRAE_APPEND_PID")));
+
+	/* Mark the initialization as if we're in instrumentation */
+	Backend_setInInstrumentation (THREADID, TRUE);
+
 	if (gethostname (hostname, sizeof(hostname)) != 0)
 		sprintf (hostname, "localhost");
 
@@ -1617,7 +1596,7 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 #endif
 
 #if defined(DEBUG)
-	fprintf (stderr, PACKAGE_NAME": DEBUG: THID=%d Backend_preInitialize (rank=%d, size=%d, config_file=\n", THREADID, me, world_size, config_file);
+	fprintf (stderr, PACKAGE_NAME": DEBUG: THID=%d Backend_preInitialize (rank=%d, size=%d, config_file=%s)\n", THREADID, me, world_size, config_file);
 #endif
 
 	if (getenv("EXTRAE_DYNINST_RUN") != NULL)
@@ -1632,7 +1611,7 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 	else
 	{
 		if (me == 0 && !forked)
-			fprintf (stdout, "Welcome to "PACKAGE_STRING" (revision %d based on %s)\n", EXTRAE_SVN_REVISION, EXTRAE_SVN_BRANCH);
+			fprintf (stdout, "Welcome to %s\n", PACKAGE_STRING);
 	}
 
 	/* Allocate a bitmap to know which tasks are tracing */
@@ -1669,7 +1648,7 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 		exit (-1);
 	}
 
-	sprintf (new_num_omp_threads_clause, "OMP_NUM_THREADS=%d\n", numProcessors);
+	sprintf (new_num_omp_threads_clause, "OMP_NUM_THREADS=%d", numProcessors);
 	omp_value = getenv ("OMP_NUM_THREADS");
 	if (omp_value)
 	{
@@ -1699,6 +1678,7 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 		putenv (new_num_omp_threads_clause);
 		current_NumOfThreads = maximum_NumOfThreads = numProcessors;
 	}
+
 # endif /* OMPT_INSTRUMENTATION */
 
 #elif defined(SMPSS_SUPPORT) || defined(NANOS_SUPPORT) || defined (UPC_SUPPORT)
@@ -1715,10 +1695,6 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 	}
 #endif
 
-#if defined(IS_CELL_MACHINE)
-	prepare_CELLTrace_init (get_maximum_NumOfThreads());
-#endif
-
 #endif /* STANDALONE */
 
 	/* Initialize the clock */
@@ -1781,8 +1757,11 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 #if defined(MPI_SUPPORT)
 	Extrae_MPI_prepareDirectoryStructures (me, world_size);
 #else
-	Backend_createExtraeDirectory (me, FALSE);
+	/* Create temporal directory */
 	Backend_createExtraeDirectory (me, TRUE);
+
+	/* Create final directory */
+	Backend_createExtraeDirectory (me, FALSE);
 #endif
 
 	/* Allocate the buffers and trace files */
@@ -1793,13 +1772,14 @@ int Backend_preInitialize (int me, int world_size, char *config_file, int forked
 	Extrae_reallocate_CUDA_info (get_maximum_NumOfThreads());
 #endif
 
-	if (!appending)
+	if (!Extrae_getAppendingEventsToGivenPID(NULL))
 	{
 		ApplBegin_Time = TIME;
 		TRACE_EVENT (ApplBegin_Time, APPL_EV, EVT_BEGIN);
 #if !defined(IS_BG_MACHINE)
 		Extrae_AnnotateTopology (TRUE, ApplBegin_Time);
 #endif
+		TRACE_EVENT (ApplBegin_Time, CPU_EVENT_INTERVAL_EV, MinimumCPUEventTime);
 
 	/* Initialize Tracing Mode related variables */
 	if (forked)
@@ -1978,12 +1958,12 @@ static int GetTraceOptions (void)
 	return options;
 }
 
-int Backend_postInitialize (int rank, int world_size, unsigned init_event, unsigned long long InitTime, unsigned long long EndTime, char **node_list)
+int Backend_postInitialize (int rank, int world_size, unsigned init_event,
+	unsigned long long InitTime, unsigned long long EndTime, char **node_list)
 {
 	unsigned u;
 	int i;
 	unsigned long long *StartingTimes=NULL, *SynchronizationTimes=NULL;
-	int appending = getenv ("EXTRAE_APPEND_PID") != NULL;
 
 #if defined(DEBUG)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: THID=%d Backend_postInitialize (rank=%d, size=%d, syn_init_time=%llu, syn_fini_time=%llu\n", THREADID, rank, world_size, InitTime, EndTime);
@@ -2044,7 +2024,7 @@ int Backend_postInitialize (int rank, int world_size, unsigned init_event, unsig
 	}
 #endif
 
-	if ((!appending) && (init_event != 0))
+	if ((!Extrae_getAppendingEventsToGivenPID(NULL)) && (init_event != 0))
 	{
 		/* Add initialization begin and end events */
 		TRACE_MPIINITEV (InitTime, init_event, EVT_BEGIN,
@@ -2063,24 +2043,30 @@ int Backend_postInitialize (int rank, int world_size, unsigned init_event, unsig
 	/* HSG force a write to disk! */
 	Buffer_Flush(TRACING_BUFFER(THREADID));
 
-	if (mpitrace_on && !CheckForControlFile && !CheckForGlobalOpsTracingIntervals)
+	if (mpitrace_on &&
+	  !Extrae_getCheckControlFile() &&
+	  !Extrae_getCheckForGlobalOpsTracingIntervals())
 	{
 		if (rank == 0)
-			fprintf (stdout, PACKAGE_NAME": Successfully initiated with %d tasks\n\n", world_size);
+			fprintf (stdout, PACKAGE_NAME": Successfully initiated with %d tasks and %d threads\n\n", world_size, Backend_getNumberOfThreads());
 	}
-	else if (mpitrace_on && CheckForControlFile && !CheckForGlobalOpsTracingIntervals)
+	else if (mpitrace_on &&
+	  Extrae_getCheckControlFile() &&
+	  !Extrae_getCheckForGlobalOpsTracingIntervals())
 	{
 		if (rank == 0)
-			fprintf (stdout, PACKAGE_NAME": Successfully initiated with %d tasks BUT disabled by EXTRAE_CONTROL_FILE\n\n", world_size);
+			fprintf (stdout, PACKAGE_NAME": Successfully initiated with %d tasks and %d threads BUT disabled by EXTRAE_CONTROL_FILE\n\n", world_size, Backend_getNumberOfThreads());
 
 		/* Just disable the tracing until the control file is created */
 		Extrae_shutdown_Wrapper();
 		mpitrace_on = 0;		/* Disable full tracing. It will allow us to know if files must be deleted or kept */
 	}
-	else if (mpitrace_on && !CheckForControlFile && CheckForGlobalOpsTracingIntervals)
+	else if (mpitrace_on &&
+	  !Extrae_getCheckControlFile() &&
+	  Extrae_getCheckForGlobalOpsTracingIntervals())
 	{
 		if (rank == 0)
-			fprintf (stdout, PACKAGE_NAME": Successfully initiated with %d tasks BUT disabled by EXTRAE_CONTROL_GLOPS\n\n", world_size);
+			fprintf (stdout, PACKAGE_NAME": Successfully initiated with %d tasks and %d threads BUT disabled by EXTRAE_CONTROL_GLOPS\n\n", world_size, Backend_getNumberOfThreads());
 		Extrae_shutdown_Wrapper();
 	}
 
@@ -2092,8 +2078,16 @@ int Backend_postInitialize (int rank, int world_size, unsigned init_event, unsig
 	if (requestedIOInstrumentation)
 		Extrae_set_trace_io (TRUE);
 
+	/* Enable system call instrumentation if requested */
+	if (requestedSysCallInstrumentation)
+	  Extrae_set_trace_syscall (TRUE);
+
 	/* Enable sampling capabilities */
-	setSamplingEnabled (TRUE);
+	Extrae_setSamplingEnabled (TRUE);
+
+#if defined(ENABLE_PEBS_SAMPLING)
+	Extrae_IntelPEBS_enable(TRUE);
+#endif
 
 	/* We leave... so, we're no longer in instrumentatin from this point */
 	for (u = 0; u < get_maximum_NumOfThreads(); u++)
@@ -2101,6 +2095,9 @@ int Backend_postInitialize (int rank, int world_size, unsigned init_event, unsig
 
 	EXTRAE_SET_INITIALIZED(TRUE);
 
+	/* Mark the finalization of init as if we're not in instrumentation */
+	Backend_setInInstrumentation (THREADID, FALSE);
+
 	return TRUE;
 }
 
@@ -2114,11 +2111,12 @@ void Backend_Finalize_close_files(void)
 	unsigned thread;
 
 	for (thread = 0; thread < get_maximum_NumOfThreads(); thread++)
-		Backend_Finalize_close_mpits( getpid(), thread, FALSE );
+		Backend_Finalize_close_mpits (getpid(), thread, FALSE);
 }
 
 static void Backend_Finalize_close_mpits (pid_t pid, int thread, int append)
 {
+	int r;
 	char trace[TMP_DIR];
 	char tmp_name[TMP_DIR];
 	unsigned initialTASKID;
@@ -2131,23 +2129,41 @@ static void Backend_Finalize_close_mpits (pid_t pid, int thread, int append)
 		sprintf (hostname, "localhost");
 
 	/* Note! If the instrumentation was initialized by Extrae_init, the TASKID
-	   as that moment was 0, independently if MPI or PACX has run */
+	   as that moment was 0, independently if MPI has run */
 	initialTASKID = Extrae_get_initial_TASKID();
 
 	Buffer_Close(TRACING_BUFFER(thread));
 
 	/* Rename MPIT file (PID from origin is getpid(), PID from destination
 	   maybe different if append == TRUE */
-	FileName_PTT(tmp_name, Get_TemporalDir(initialTASKID), appl_name, hostname,
-	  getpid(), initialTASKID, thread, EXT_TMP_MPIT);
-	FileName_PTT(trace, Get_FinalDir(TASKID), appl_name, hostname, pid, TASKID,
-	  thread, EXT_MPIT);
-	if (!append)
-		rename_or_copy (tmp_name, trace); 
+	if (append)
+	{
+		/* When appending, we add information into an .mpit rather than into a .ttmp */
+		/* In this case, we have to honor the original PID (given by param) */
+		FileName_PTT(tmp_name, Get_TemporalDir(initialTASKID), appl_name, hostname,
+		  getpid(), initialTASKID, thread, EXT_TMP_MPIT);
+		FileName_PTT(trace, Get_FinalDir(TASKID), appl_name, hostname, pid,
+		  TASKID, thread, EXT_MPIT);
+	}
 	else
-		append_from_to_file (tmp_name, trace);
-	fprintf (stdout,
-	  PACKAGE_NAME": Intermediate raw trace file created : %s\n", trace);
+	{
+		/* When in default (not appending) we create a .ttmp and then rename into .mpit */
+		FileName_PTT(tmp_name, Get_TemporalDir(initialTASKID), appl_name, hostname,
+		  getpid(), initialTASKID, thread, EXT_TMP_MPIT);
+		FileName_PTT(trace, Get_FinalDir(TASKID), appl_name, hostname, getpid(),
+		  TASKID, thread, EXT_MPIT);
+	}
+	if (!append)
+		r = rename_or_copy (tmp_name, trace); 
+	else
+		r = append_from_to_file (tmp_name, trace);
+
+	if (r == 0)
+		fprintf (stdout,
+		  PACKAGE_NAME": Intermediate raw trace file created : %s\n", trace);
+	else
+		fprintf (stdout,
+		  PACKAGE_NAME": Intermediate raw trace was NOT created : %s\n", trace);
 
 	/* Rename SAMPLE file, if it exists */
 #if defined(SAMPLING_SUPPORT)
@@ -2160,9 +2176,14 @@ static void Backend_Finalize_close_mpits (pid_t pid, int thread, int append)
 
 		FileName_PTT(trace, Get_FinalDir(TASKID), appl_name, hostname, pid,
 		  TASKID, thread, EXT_SAMPLE);
-		rename_or_copy (tmp_name, trace);
-		fprintf (stdout,
-		  PACKAGE_NAME": Intermediate raw sample file created : %s\n", trace);
+		r = rename_or_copy (tmp_name, trace);
+
+		if (r == 0)
+			fprintf (stdout,
+			  PACKAGE_NAME": Intermediate raw sample file created : %s\n", trace);
+		else
+			fprintf (stdout,
+			  PACKAGE_NAME": Intermediate raw sample was NOT created : %s\n", trace);
 	}
 	else
 	{
@@ -2176,9 +2197,14 @@ static void Backend_Finalize_close_mpits (pid_t pid, int thread, int append)
     if (file_exists(tmp_name)){
         FileName_PTT(trace, Get_FinalDir(initialTASKID), appl_name, hostname,
 		  pid, initialTASKID, thread, EXT_SYM);
-        rename_or_copy(tmp_name, trace);
-    	fprintf (stdout,
-    	  PACKAGE_NAME": Intermediate raw sym file created : %s\n", trace);
+        r = rename_or_copy(tmp_name, trace);
+
+		if (r == 0)
+	    	fprintf (stdout,
+	    	  PACKAGE_NAME": Intermediate raw sym file created : %s\n", trace);
+		else
+  	  		fprintf (stdout,
+   	 		  PACKAGE_NAME": Intermediate raw sym was NOT created : %s\n", trace);
     }
 }
 
@@ -2214,14 +2240,14 @@ int Extrae_Flush_Wrapper (Buffer_t *buffer)
 		FlushEv_Begin.time = TIME;
 		FlushEv_Begin.event = FLUSH_EV;
 		FlushEv_Begin.value = EVT_BEGIN;
-		HARDWARE_COUNTERS_READ (THREADID, FlushEv_Begin, TRUE);
+		HARDWARE_COUNTERS_READ (THREADID, FlushEv_Begin, Extrae_Flush_Wrapper_getCounters());
 
 		Buffer_Flush (buffer);
 
 		FlushEv_End.time = TIME;
 		FlushEv_End.event = FLUSH_EV;
 		FlushEv_End.value = EVT_END;
-		HARDWARE_COUNTERS_READ (THREADID, FlushEv_End, TRUE);
+		HARDWARE_COUNTERS_READ (THREADID, FlushEv_End, Extrae_Flush_Wrapper_getCounters());
 
 		BUFFER_INSERT (THREADID, buffer, FlushEv_Begin);
 #if !defined(IS_BG_MACHINE)
@@ -2253,7 +2279,10 @@ int Extrae_Flush_Wrapper (Buffer_t *buffer)
 void Backend_Finalize (void)
 {
 	unsigned thread;
-	int appending = getenv ("EXTRAE_APPEND_PID") != NULL;
+
+#if defined(ENABLE_PEBS_SAMPLING)
+	Extrae_IntelPEBS_disable();
+#endif
 
 #if defined(CUDA_SUPPORT)
 	Extrae_CUDA_fini ();
@@ -2263,7 +2292,11 @@ void Backend_Finalize (void)
 	Extrae_OpenCL_fini ();
 #endif
 
-	if (!appending)
+#if defined(OMP_SUPPORT) && defined(OMPT_INSTRUMENTATION)
+        ompt_finalize ();
+#endif
+
+	if (!Extrae_getAppendingEventsToGivenPID(NULL))
 	{
 #if defined(HAVE_ONLINE)
 		/* Stop the analysis threads and flush the online buffers */
@@ -2280,14 +2313,9 @@ void Backend_Finalize (void)
 		Extrae_set_trace_malloc (FALSE);
 
 		/* Stop sampling right now */
-		setSamplingEnabled (FALSE);
+		Extrae_setSamplingEnabled (FALSE);
 		unsetTimeSampling ();
 
-		/* Write files back to disk , 1st part will include flushing events*/
-		for (thread = 0; thread < get_maximum_NumOfThreads(); thread++) 
-			if (TRACING_BUFFER(thread) != NULL)
-				Buffer_ExecuteFlushCallback (TRACING_BUFFER(thread));
-
 		if (THREADID == 0) 
 		{
 			Extrae_getrusage_Wrapper ();
@@ -2298,21 +2326,30 @@ void Backend_Finalize (void)
 		Extrae_AnnotateTopology (TRUE, TIME);
 #endif
 
-		/* Final write files to disk, include renaming of the filenames */
-		for (thread = 0; thread < get_maximum_NumOfThreads(); thread++)
+		/* Write files back to disk , 1st part will include flushing events*/
+		for (thread = 0; thread < get_maximum_NumOfThreads(); thread++) 
 		{
 			/* Prevent writing performance counters from another thread */
 			if (thread != THREADID)
 				Extrae_Flush_Wrapper_setCounters (FALSE);
 
+			if (TRACING_BUFFER(thread) != NULL)
+				Buffer_ExecuteFlushCallback (TRACING_BUFFER(thread));
+
+			Extrae_Flush_Wrapper_setCounters (TRUE);
+		}
+
+		/* Final write files to disk, include renaming of the filenames,
+		   do not need counters here */
+		Extrae_Flush_Wrapper_setCounters (FALSE);
+		for (thread = 0; thread < get_maximum_NumOfThreads(); thread++)
+		{
 			if (TRACING_BUFFER(thread) != NULL)
 			{
 				TRACE_EVENT (TIME, APPL_EV, EVT_END);
 				Buffer_ExecuteFlushCallback (TRACING_BUFFER(thread));
 				Backend_Finalize_close_mpits (getpid(), thread, FALSE);
 			}
-
-			Extrae_Flush_Wrapper_setCounters (TRUE);
 		}
 	
 		/* Free allocated memory */
@@ -2322,6 +2359,9 @@ void Backend_Finalize (void)
 
 			for (thread = 0; thread < get_maximum_NumOfThreads(); thread++)
 			{
+#if defined(PTHREAD_SUPPORT)
+				pThreads[thread] = (pthread_t)0;
+#endif
 				Buffer_Free (TRACING_BUFFER(thread));
 				TRACING_BUFFER(thread) = NULL;
 #if defined(SAMPLING_SUPPORT)
@@ -2329,6 +2369,8 @@ void Backend_Finalize (void)
 				SAMPLING_BUFFER(thread) = NULL;
 #endif
 			}
+			xfree(LastCPUEmissionTime);
+			xfree(LastCPUEvent);
 			xfree(TracingBuffer);
 #if defined(SAMPLING_SUPPORT)
 			xfree(SamplingBuffer);
@@ -2350,6 +2392,9 @@ void Backend_Finalize (void)
 
 		mpitrace_on = FALSE; /* Turn off tracing now */
 
+		/* Mark as already finalized, thus avoiding further deinits */
+		Extrae_set_is_initialized (EXTRAE_NOT_INITIALIZED);
+
 #if defined(EMBED_MERGE_IN_TRACE)
 		/* Launch the merger */
 		if (MergeAfterTracing)
@@ -2374,7 +2419,7 @@ void Backend_Finalize (void)
 #if defined(MPI_SUPPORT)
 			sprintf (tmp, "%s", Extrae_core_get_mpits_file_name());
 #else
-			sprintf (tmp, "%s/%s.mpits", final_dir, appl_name);
+			sprintf (tmp, "%s/%s%s", final_dir, appl_name, EXT_MPITS);
 #endif
 
 			Read_MPITS_file (tmp, &ptask, FileOpen_Default, TASKID);
@@ -2388,24 +2433,33 @@ void Backend_Finalize (void)
 	}
 	else
 	{
+		int pid;
+		Extrae_getAppendingEventsToGivenPID (&pid);
 		Buffer_Flush(TRACING_BUFFER(THREADID));
-		Backend_Finalize_close_mpits (atoi(getenv("EXTRAE_APPEND_PID")), THREADID, TRUE);
+		for (thread = 0; thread < get_maximum_NumOfThreads(); thread++) 
+			Backend_Finalize_close_mpits (pid, thread, TRUE);
 		remove_temporal_files ();
 	}
 }
 
 //static int Extrae_inInstrumentation = FALSE;
 static int *Extrae_inInstrumentation = NULL;
+static int *Extrae_inSampling = NULL;
 
 int Backend_inInstrumentation (unsigned thread)
 {
-	if (Extrae_inInstrumentation != NULL)
-		return Extrae_inInstrumentation[thread];
+	if ((Extrae_inInstrumentation != NULL) && (Extrae_inSampling != NULL))
+		return (Extrae_inInstrumentation[thread] || Extrae_inSampling[thread]);
 	else
 		return FALSE;
-	/* return Extrae_inInstrumentation; */
 }
 
+void Backend_setInSampling (unsigned thread, int insampling)      
+{                                                                               
+	  if (Extrae_inSampling != NULL)                                         
+			    Extrae_inSampling[thread] = insampling;                       
+}                                                                               
+
 void Backend_setInInstrumentation (unsigned thread, int ininstrumentation)
 {
 	if (Extrae_inInstrumentation != NULL)
@@ -2414,14 +2468,20 @@ void Backend_setInInstrumentation (unsigned thread, int ininstrumentation)
 
 void Backend_ChangeNumberOfThreads_InInstrumentation (unsigned nthreads)
 {
-	Extrae_inInstrumentation = (int*) realloc (Extrae_inInstrumentation,
-	  sizeof(int)*nthreads);
+	Extrae_inInstrumentation = (int*) realloc (Extrae_inInstrumentation, sizeof(int)*nthreads);
 	if (Extrae_inInstrumentation == NULL)
 	{
 		fprintf (stderr, PACKAGE_NAME
 		  ": Failed to allocate memory for inInstrumentation structure\n");
 		exit (-1);
 	}
+	Extrae_inSampling = (int*) realloc (Extrae_inSampling, sizeof(int)*nthreads);
+	if (Extrae_inSampling == NULL)
+	{
+		fprintf (stderr, PACKAGE_NAME
+		  ": Failed to allocate memory for inSampling structure\n");
+		exit (-1);
+	}
 }
 
 void Backend_Enter_Instrumentation (int Nevents)
@@ -2434,6 +2494,43 @@ void Backend_Enter_Instrumentation (int Nevents)
 
 	Backend_setInInstrumentation (thread, TRUE);
 
+	/* Check if we have to fill the sampling buffer */
+#if defined(SAMPLING_SUPPORT)
+	if (Extrae_get_DumpBuffersAtInstrumentation())
+		if (Buffer_IsFull (SAMPLING_BUFFER(THREADID)))
+		{
+			event_t FlushEv_Begin, FlushEv_End;
+
+			/* Disable sampling first, and then reestablish whether it was set */
+			int prev = Extrae_isSamplingEnabled();
+			Extrae_setSamplingEnabled (FALSE);
+
+			/* Get time now, to mark in the instrumentation buffer the begin of the flush */
+			FlushEv_Begin.time = TIME;
+			FlushEv_Begin.event = FLUSH_EV;
+			FlushEv_Begin.value = EVT_BEGIN;
+			HARDWARE_COUNTERS_READ (THREADID, FlushEv_Begin,
+			  Extrae_Flush_Wrapper_getCounters());
+
+			/* Actually flush buffer */
+			Buffer_Flush(SAMPLING_BUFFER(THREADID));
+
+			/* Get time now, to mark in the instrumentation buffer the end of the flush */
+			FlushEv_End.time = TIME;
+			FlushEv_End.event = FLUSH_EV;
+			FlushEv_End.value = EVT_END;
+			HARDWARE_COUNTERS_READ (THREADID, FlushEv_End,
+			  Extrae_Flush_Wrapper_getCounters());
+
+			/* Add events into instrumentation buffer */
+			BUFFER_INSERT (THREADID, TRACING_BUFFER(THREADID), FlushEv_Begin);
+			BUFFER_INSERT (THREADID, TRACING_BUFFER(THREADID), FlushEv_End);
+
+			/* Reestablish sampling */
+			Extrae_setSamplingEnabled (prev);
+		}
+#endif
+
 	/* Check whether we will fill the buffer soon (or now) */
 	if (Buffer_RemainingEvents(TracingBuffer[thread]) <= Nevents)
 		Buffer_ExecuteFlushCallback (TracingBuffer[thread]);
@@ -2464,6 +2561,11 @@ void Backend_Leave_Instrumentation (void)
 	if (!mpitrace_on)
 		return;
 
+	if (PENDING_TRACE_CPU_EVENT(thread, LAST_READ_TIME))
+	{
+		Extrae_AnnotateCPU(LAST_READ_TIME);
+	}
+
 	/* Change trace mode? (issue from API) */
 	if (PENDING_TRACE_MODE_CHANGE(thread) && MPI_Deepness[thread] == 0)
 		Trace_Mode_Change(thread, LAST_READ_TIME);
@@ -2695,6 +2797,8 @@ void Backend_updateTaskID (void)
 	/* Rename SYM file, if it exists, per thread */
 	for (thread = 0; thread < get_maximum_NumOfThreads(); thread++)
 	{
+		int r;
+
 		FileName_PTT(file1, Get_TemporalDir(Extrae_get_initial_TASKID()),
 		  appl_name, hostname, getpid(), Extrae_get_initial_TASKID(), thread,
 		  EXT_SYM);
@@ -2706,9 +2810,11 @@ void Backend_updateTaskID (void)
 			/* Remove file if it already exists, and then copy the "new" version */
 			if (file_exists (file2))
 				if (!unlink (file2) == 0)
-					fprintf (stderr, PACKAGE_NAME": Cannot unlink file: %s, symbols will be corrupted!\n", file2);
+					fprintf (stderr, PACKAGE_NAME": Cannot unlink symbolic file: %s, symbols will be corrupted!\n", file2);
 
-			rename_or_copy (file1, file2); 
+			r = rename_or_copy (file1, file2); 
+			if (r < 0)
+				fprintf (stderr, PACKAGE_NAME": Error copying symbolicfile %s into %s!\n", file1, file2);
 		}
 	}
 	/* NOTE: we skip renaming the MPIT and the SAMPLE files because there are
