@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/OMP/ibm-xlsmp-1.6.c $
- | @last_commit: $Date: 2015-04-21 16:45:09 +0200 (mar, 21 abr 2015) $
- | @version:     $Revision: 3260 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: ibm-xlsmp-1.6.c 3260 2015-04-21 14:45:09Z harald $";
-
 #ifdef HAVE_DLFCN_H
 # define __USE_GNU
 # include <dlfcn.h>
